#include "testApp.h"

void drawHead(float t) {
  ofPushMatrix();
  ofRotateZ(t);
  
  ofSetColor(240, 240, 240, 255);
  ofFill();
  ofRect(-50, 0, 100, 30);
  ofBeginShape();
  ofVertex(0, -100);
  ofVertex(-35, -50);
  ofVertex(-50, 0);
  ofVertex(-35, 50);
  ofVertex(0, 100);
  
  ofVertex(-100, 75);
  ofVertex(-175, 0);
  ofVertex(-100, -75);
  ofEndShape(true);
  
  ofNoFill();
  ofRect(-50, 0, 100, 30);
  ofBeginShape();
  ofVertex(0, -100);
  ofVertex(-35, -50);
  ofVertex(-50, 0);
  ofVertex(-35, 50);
  ofVertex(0, 100);
  
  ofVertex(-100, 75);
  ofVertex(-175, 0);
  ofVertex(-100, -75);
  ofEndShape(true);
  
  ofSetColor(32, 32, 32, 255);
  ofSetLineWidth(2);
  ofLine(-80, -40, -100, -60);
  ofLine(-100, -40, -80, -60);
  ofSetLineWidth(1);
}

void drawRib(float t, float s) {
  ofPushMatrix();
  ofRotateY(-t*3);
  ofRotateZ(t);
  
  ofSetColor(240, 240, 240, 255);
  ofPushMatrix();
  ofScale(s, s, 0);
  ofFill();
  ofRect(15, 0, 10, 60);
  ofRect(15, 0, 30, 10);
  ofNoFill();
  ofRect(15, 0, 10, 60);
  ofRect(15, 0, 30, 10);
  ofPopMatrix();
  ofTranslate(30*s, 0);
}

void drawTail(float t) {
  ofPushMatrix();
  ofRotateZ(t);
  
  ofSetColor(240, 240, 240, 255);
  ofPushMatrix();
  
  ofFill();
  ofRect(15, 0, 30, 10);
  ofNoFill();
  ofRect(15, 0, 30, 10);
  
  ofFill();
  ofBeginShape();
  ofVertex(43, -50);
  ofVertex(35, -25);
  ofVertex(31, 0);
  ofVertex(35, 25);
  ofVertex(43, 50);
  
  ofVertex(18, 37);
  ofVertex(0, 0);
  ofVertex(18, -37);
  ofEndShape();
  ofPopMatrix();
  
  ofNoFill();
  ofBeginShape();
  ofVertex(43, -50);
  ofVertex(35, -25);
  ofVertex(31, 0);
  ofVertex(35, 25);
  ofVertex(43, 50);
  
  ofVertex(18, 37);
  ofVertex(0, 0);
  ofVertex(18, -37);
  ofEndShape();
  ofPopMatrix();
}

void drawFish(float t) {
  ofPushMatrix();
  ofScale(-1, 1, 1);
  drawHead(5 * sin(t+6));  
  drawRib(15 * sin(t+5), 2.75);
  drawRib(15 * sin(t+4), 2.25);
  drawRib(15 * sin(t+3), 2.0);
  drawRib(15 * sin(t+2), 1.75);
  drawRib(15 * sin(t+1), 1.5);
  drawTail(15 * sin(t+0));
  ofPopMatrix();
  
  ofPopMatrix();
  ofPopMatrix();
  ofPopMatrix();
  ofPopMatrix();
  ofPopMatrix();
  ofPopMatrix();
}

//--------------------------------------------------------------
void testApp::setup(){
  ofBackground(32, 32, 32);
  ofEnableSmoothing();
  ofSetRectMode(OF_RECTMODE_CENTER);
  
  fishPos.set(ofGetWidth()/2, ofGetHeight()/2);
  rz = ry = 0;
  
  speed = MIN_MULT;
  t = 0;
}

//--------------------------------------------------------------
void testApp::update(){
  // t += MAX_SPEED * (float)mouseX/ofGetWidth();
  t += MAX_SPEED * speed;
  if(speed > MIN_MULT) speed -= 0.001;
  
  float s = speed * 0.001;
  
  fishPos.x = (1-s)*fishPos.x + s*mouseX;
  fishPos.y = (1-s)*fishPos.y + s*mouseY;
  
  rz = 0.999*rz + 0.001*CLAMP(atan2(mouseY - fishPos.y, mouseX - fishPos.x)*(180.0/PI), -45, 45);
  
  ry = ofMap(mouseX - fishPos.x, -300, 300, 180, 0, true);
}

//--------------------------------------------------------------
void testApp::draw(){
  ofPushMatrix();
  ofTranslate(fishPos.x, fishPos.y);
  ofScale(0.4, 0.4, 0);
  ofRotateY(ry);
  ofRotateZ(rz);
  
  drawFish(t);
  
  ofPopMatrix();
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
  speed += 1.0;
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

